/*
 * NewsBlob - Live RSS news reader.
 * Copyright 2009 Lars Torger Høye.
 *
 * This file is part of NewsBlob.
 *
 * NewsBlob is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * NewsBlob is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NewsBlob.  If not, see <http://www.gnu.org/licenses/>.
 */

package newsblob;

import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import newsblob.gui.AboutFrm;
import newsblob.gui.AddNewsfeedFrm;

/**
 *
 * @author Lars Torger Høye
 */
public class NewsBlobMenu {

    private PopupMenu m_menu;

    private static NewsBlobMenu m_instance;

    public NewsBlobMenu() {
        m_menu = new PopupMenu();

        MenuItem item = new MenuItem("Add newsfeed...");
        class AddNewsfeedListener implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                AddNewsfeedFrm newsFeed = new AddNewsfeedFrm();
                newsFeed.setVisible(true);
            }
        }
        item.addActionListener(new AddNewsfeedListener());
        m_menu.add(item);

        m_menu.addSeparator();

        item = new MenuItem("About...");
        class AboutListener implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                new AboutFrm().setVisible(true);
            }
        }
        item.addActionListener(new AboutListener());
        m_menu.add(item);

        item = new MenuItem("Exit");
        class ExitListener implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                NewsBlob.getInstance().exit();
            }
        }
        item.addActionListener(new ExitListener());
        m_menu.add(item);
    }

    public static NewsBlobMenu getInstance() {
        if(m_instance == null)
            m_instance = new NewsBlobMenu();
        return m_instance;
    }

    public void add(Menu menuItem) {
        if(m_menu.getItemCount() == 4)
            m_menu.insertSeparator(1);
        m_menu.insert(menuItem, 2);
    }

    public void remove(Menu menuItem) {
        m_menu.remove(menuItem);
        if(m_menu.getItemCount() == 5)
            m_menu.remove(1);
    }

    public PopupMenu getMenu() {
        return m_menu;
    }
}
