/*
 * NewsBlob - Live RSS news reader.
 * Copyright 2009 Lars Torger Høye.
 *
 * This file is part of NewsBlob.
 *
 * NewsBlob is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * NewsBlob is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NewsBlob.  If not, see <http://www.gnu.org/licenses/>.
 */

package newsblob.blob;

import java.awt.AWTException;
import newsblob.feed.Newsfeed;
import newsblob.feed.NewsfeedThread;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.TrayIcon.MessageType;
import java.util.ArrayList;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;
import newsblob.feed.article.Article;

/**
 *
 * @author Lars Torger Høye
 */
public class Blob {

    private TrayIcon m_icon;
    private ArrayList<NewsfeedThread> m_newsThreads;

    private boolean m_close = false;

    public Blob(TrayIcon icon) {
        m_icon = icon;
        m_newsThreads = new ArrayList<NewsfeedThread>();
    }

    public void add(Newsfeed newsfeed) throws ParserConfigurationException, XPathExpressionException, Exception {
        if(newsfeed == null) return;

        for (NewsfeedThread newsThread : m_newsThreads) {
            if(newsThread.getNewsfeed().equals(newsfeed)) {
                throw new Exception("This feed already exists");
            }
        }

        NewsfeedThread newsTread = new NewsfeedThread(newsfeed, this);
        newsTread.start();
        m_newsThreads.add(newsTread);
    }

    public void remove(Newsfeed newsfeed) {
        NewsfeedThread removeThread = null;
        for (NewsfeedThread newsThread : m_newsThreads) {
            if(newsThread.getNewsfeed().getURI().equals(newsfeed.getURI()))
                removeThread = newsThread;
        }
        if(removeThread != null) {
            removeThread.close();
            m_newsThreads.remove(removeThread);
        }
    }

    public void removeAll() {
        for (NewsfeedThread newsfeed : m_newsThreads) {
            newsfeed.close();
        }
        m_newsThreads.clear();
    }

    public void close() {
        removeAll();
        m_close = true;
    }

    public synchronized void print(Newsfeed newsfeed, Article article) throws InterruptedException {

        if(m_close) return;

        SystemTray sysTray = SystemTray.getSystemTray();
        LinkListener listener = new LinkListener(article.getLink());

        try {
            sysTray.add(m_icon);
            m_icon.addActionListener(listener);

            //Display message
            m_icon.displayMessage(article.getTitle()+" - "+newsfeed.getTitle(), article.getFilteredDescription(), MessageType.NONE);
            //System.out.println(new Date()+"\n"+article.getTitle()+"\n"+article.getFilteredDescription()+"\n"+article.getLink()+"\n");  //Debugging

            Thread.sleep(newsfeed.getDisplayTime()*1000);
        }
        catch(AWTException ie) {}
        finally {
            m_icon.removeActionListener(listener);
            sysTray.remove(m_icon);
        }
    }
}
