/*
 * NewsBlob - Live RSS news reader.
 * Copyright 2009 Lars Torger Høye.
 *
 * This file is part of NewsBlob.
 *
 * NewsBlob is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * NewsBlob is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NewsBlob.  If not, see <http://www.gnu.org/licenses/>.
 */

package newsblob.feed.file;

import newsblob.feed.*;
import newsblob.blob.Blob;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import javax.swing.JOptionPane;

/**
 *
 * @author Lars Torger Høye
 */
public class NewsfeedsFile {

    private File m_file;
    private File m_backup;

    private NewsfeedFormat m_format;

    public NewsfeedsFile(String file) throws FileNotFoundException, IOException {
        m_file = new File(file);
        m_backup = new File(m_file.getParentFile(), m_file.getName()+".bak");
        m_backup.deleteOnExit();    //Delete backup if we exit successfully.

        BufferedReader in = new BufferedReader(new FileReader(m_file));
        String line;
        String format = null;
        while((line = in.readLine()) != null && format == null) {
            String tLine = line.trim().toLowerCase();

            if(tLine.startsWith("format")) {
                String[] splitLine = tLine.split("=");
                m_format = new NewsfeedFormat(splitLine[1]);
            }
        }
        in.close();
    }

    public synchronized void load(Blob blob) {
        try {
            BufferedReader in = new BufferedReader(new FileReader(m_file));
            String line;

            while((line = in.readLine()) != null) {
                String tLine = line.trim();

                if(!tLine.equals("") && !tLine.startsWith("#") && !tLine.toLowerCase().startsWith("format")) {
                    blob.add(m_format.decodeNewsfeed(tLine));
                }
            }
            in.close();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Unable to load newsfeeds from disk.", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public synchronized void add(Newsfeed newsfeed) throws FileNotFoundException, IOException {
        backup();

        PrintWriter out = new PrintWriter(new FileOutputStream(m_file, true));
        out.print(m_format.encodeNewsfeed(newsfeed));
        out.close();
    }

    public synchronized void remove(Newsfeed newsfeed) throws FileNotFoundException, IOException {
        backup();

        BufferedReader in = new BufferedReader(new FileReader(m_backup));
        PrintWriter out = new PrintWriter(new FileOutputStream(m_file));

        String line;
        while((line = in.readLine()) != null) {
            if(line.indexOf(newsfeed.getURI()) == -1)
                out.println(line);
        }
        in.close();
        out.close();
    }
    
    private void backup() throws FileNotFoundException, IOException {
        DataInputStream in = new DataInputStream(new FileInputStream(m_file));
        DataOutputStream out = new DataOutputStream(new FileOutputStream(m_backup));
        byte[] buffer = new byte[0x8000];   //32k

        int eof;
        while((eof = in.read(buffer)) != -1) {
            out.write(buffer, 0, eof);
        }
        in.close();
        out.close();
    }
}
