/*
 * NewsBlob - Live RSS news reader.
 * Copyright 2009 Lars Torger Høye.
 *
 * This file is part of NewsBlob.
 *
 * NewsBlob is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * NewsBlob is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with NewsBlob.  If not, see <http://www.gnu.org/licenses/>.
 */

package newsblob.gui;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;
import newsblob.NewsBlob;
import newsblob.feed.Newsfeed;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 *
 * @author Lars Torger Høye
 */
public class AddNewsfeedFrm extends javax.swing.JFrame {

    private final String m_updateFrequencyTip = "Update frequency in minutes.";
    private final String m_displayTimeTip = "Display time in seconds.";

    private DocumentBuilder m_documentBuilder;
    private XPath m_xPath;
    private XPathExpression m_xPathExpression;

    private String m_newsfeedTitle;
    private String m_newsfeedTitleColor = "red";

    /** Creates new form Add */
    public AddNewsfeedFrm() {
        initComponents();
    }

    private String loadRSS(String uri) {
        uri = uri.trim();
        if(uri.equals("")) return null;

        try {
            Document doc = m_documentBuilder.parse(uri);
            //NodeList result = (NodeList)m_xPathExpression.evaluate(doc, XPathConstants.NODESET);
            Node result = (Node)m_xPathExpression.evaluate(doc, XPathConstants.NODE);

            return result.getTextContent();
        }
        catch(Exception e) {}
        
        return null;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_titleLabel = new javax.swing.JLabel();
        m_uriLabel = new javax.swing.JLabel();
        m_addBtn = new javax.swing.JButton();
        m_ufLabel = new javax.swing.JLabel();
        m_updateFrequenzyTxt = new javax.swing.JTextField();
        m_dtLabel = new javax.swing.JLabel();
        m_displayTimeTxt = new javax.swing.JTextField();
        m_recentNewsCBox = new javax.swing.JCheckBox();
        m_uriTxt = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Add newsfeed.");
        setIconImage(new ImageIcon("icon.png").getImage());
        setLocationByPlatform(true);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowOpened(java.awt.event.WindowEvent evt) {
                formWindowOpened(evt);
            }
        });

        m_titleLabel.setText("<html><b>Newsfeed</b></html>");
        m_titleLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                m_titleLabelMouseClicked(evt);
            }
        });

        m_uriLabel.setText("URL:");

        m_addBtn.setText("Check");
        m_addBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_addBtnActionPerformed(evt);
            }
        });

        m_ufLabel.setText("UF:");
        m_ufLabel.setToolTipText(m_updateFrequencyTip);

        m_updateFrequenzyTxt.setText("5");
        m_updateFrequenzyTxt.setToolTipText(m_updateFrequencyTip);

        m_dtLabel.setText("DT:");
        m_dtLabel.setToolTipText(m_displayTimeTip);

        m_displayTimeTxt.setText("15");
        m_displayTimeTxt.setToolTipText(m_displayTimeTip);

        m_recentNewsCBox.setSelected(true);
        m_recentNewsCBox.setText("Display recent news.");
        m_recentNewsCBox.setEnabled(false);

        m_uriTxt.setColumns(32);
        m_uriTxt.setMaximumSize(new java.awt.Dimension(6, 20));
        m_uriTxt.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                m_uriTxtMouseClicked(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_titleLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 334, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(m_uriLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(m_addBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(m_ufLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(m_updateFrequenzyTxt, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(m_dtLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(m_displayTimeTxt, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(m_recentNewsCBox))
                            .addComponent(m_uriTxt, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_titleLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_uriLabel)
                    .addComponent(m_uriTxt, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_addBtn)
                    .addComponent(m_ufLabel)
                    .addComponent(m_updateFrequenzyTxt, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_displayTimeTxt, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_dtLabel)
                    .addComponent(m_recentNewsCBox))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowOpened(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowOpened
        getRootPane().setDefaultButton(m_addBtn);
        
        //Set up xml
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            m_documentBuilder = factory.newDocumentBuilder();

            m_xPath = XPathFactory.newInstance().newXPath();
            m_xPathExpression = m_xPath.compile("/rss/channel/title");
        } catch (Exception e) {}

        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        try {
            String text = ((String) clipboard.getData(DataFlavor.stringFlavor)).trim();
            if(text.trim().length() < 7) {    //http://
                return;
            }
            if(text.trim().substring(0, 7).equals("http://")) {
                m_newsfeedTitle = loadRSS(text);
                if(m_newsfeedTitle != null) {
                    //m_newsfeedTitleColor = "green";
                    //m_titleLabel.setText("<html><b><font color='"+m_newsfeedTitleColor+"'>"+m_newsfeedTitle+"</font></b></html>");
                    //m_uriTxt.setEnabled(false);
                    m_uriTxt.setText(text);
                    m_uriTxt.selectAll();
                    //m_addBtn.setText("Add");
                }
            }
        } catch (Exception ex) {
            Logger.getLogger(AddNewsfeedFrm.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_formWindowOpened

    private void m_addBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_addBtnActionPerformed
        String uri = m_uriTxt.getText().trim();

        if(m_addBtn.getText().equals("Add")) {
            try {
                Newsfeed newsfeed = new Newsfeed(
                        m_newsfeedTitle,
                        uri,
                        Integer.parseInt(m_updateFrequenzyTxt.getText()),
                        Integer.parseInt(m_displayTimeTxt.getText()));
                NewsBlob.getInstance().add(newsfeed);
                dispose();
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this, "Unable to add newsfeed.", "Error", JOptionPane.ERROR_MESSAGE);
            }
        } else {
            m_newsfeedTitle = loadRSS(m_uriTxt.getText());
            if(m_newsfeedTitle != null) {
                m_newsfeedTitleColor = "green";
                m_uriTxt.setEnabled(false);
                m_addBtn.setText("Add");
                m_titleLabel.setToolTipText("Newsfeed title. Click to change...");
            } else {
                m_newsfeedTitleColor = "red";
                m_newsfeedTitle = "Unknown newsfeed";
            }
            m_titleLabel.setText("<html><b><font color='"+m_newsfeedTitleColor+"'>"+m_newsfeedTitle+"</font></b></html>");
        }
    }//GEN-LAST:event_m_addBtnActionPerformed

    private void m_titleLabelMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_m_titleLabelMouseClicked
        if(m_addBtn.getText().equals("Add")) {
            NewsfeedTitleDlg dlg = new NewsfeedTitleDlg(this, true);
            dlg.setNewsfeedTitle(m_newsfeedTitle);
            dlg.setVisible(true);
            if(dlg.isOKPressed()) {
                m_newsfeedTitle = dlg.getNewsfeedTitle();
                m_titleLabel.setText("<html><b><font color='"+m_newsfeedTitleColor+"'>"+m_newsfeedTitle+"</font></b></html>");
            }
        }
    }//GEN-LAST:event_m_titleLabelMouseClicked

    private void m_uriTxtMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_m_uriTxtMouseClicked
        if(!m_uriTxt.isEnabled()) {
            m_addBtn.setText("Check");
            m_titleLabel.setToolTipText("");
            m_uriTxt.setEnabled(true);
            m_uriTxt.selectAll();
            m_uriTxt.requestFocusInWindow();
        }
    }//GEN-LAST:event_m_uriTxtMouseClicked

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new AddNewsfeedFrm().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton m_addBtn;
    private javax.swing.JTextField m_displayTimeTxt;
    private javax.swing.JLabel m_dtLabel;
    private javax.swing.JCheckBox m_recentNewsCBox;
    private javax.swing.JLabel m_titleLabel;
    private javax.swing.JLabel m_ufLabel;
    private javax.swing.JTextField m_updateFrequenzyTxt;
    private javax.swing.JLabel m_uriLabel;
    private javax.swing.JTextField m_uriTxt;
    // End of variables declaration//GEN-END:variables

}
